import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import * as path from "path";
import * as mime from "mime";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import {
  controllerCreator,
  removeSlashes,
  notUndefined
} from "../../utils";
class StaticsController {
  constructor({ inject, ...options }) {
    __publicField(this, "_sendFile");
    __publicField(this, "_options");
    __publicField(this, "files");
    this._sendFile = inject.sendFile;
    this._options = this._validateOptions(
      deepAssignWithOverwrite(
        {
          files: ["favicon.ico", "index.html"],
          methods: options.methods || {
            all: false,
            get: true
          },
          paths: {
            route: "",
            source: "./"
          }
        },
        options
      )
    );
    this.files = this._createFiles();
  }
  addRoutes(router, middlewares = []) {
    const { methods } = this._options;
    const use = methods.all ? ["all"] : Object.keys(methods).reduce((acc, name) => {
      const methodName = name;
      if (methods[methodName]) {
        acc.push(methodName);
      }
      return acc;
    }, []);
    Object.keys(this.files).forEach((route) => {
      const file = this.files[route];
      const fileMiddleware = this._getMiddleware(file);
      use.forEach(
        (method) => this._addRoute({ router, method, file, fileMiddleware, middlewares })
      );
    });
    return router;
  }
  get options() {
    return { ...this._options };
  }
  _getMiddleware(file) {
    return (_, res, next) => {
      const extension = path.parse(file.path).ext.substring(1);
      const headers = {
        "Content-Type": mime.getType(extension) || "text/html",
        ...file.headers
      };
      Object.entries(headers).forEach(([key, value]) => {
        res.setHeader(key, value);
      });
      this._sendFile({
        res,
        filepath: file.path,
        next
      });
    };
  }
  _addRoute({
    router,
    method,
    file,
    fileMiddleware,
    middlewares
  }) {
    const { route } = file;
    router[method](route, [...middlewares, fileMiddleware]);
  }
  _validateOptions(options) {
    if (!options.files || !options.files.length) {
      throw new Error("You need to specify a list of files");
    }
    if (!options.methods) {
      throw new Error("You need to specify which HTTP methods are allowed for the files");
    }
    const methods = Object.keys(options.methods);
    const atLeastOne = methods.some((method) => options.methods[method]);
    if (!atLeastOne) {
      throw new Error("You need to enable at least one HTTP method to serve the files");
    }
    const allowedMethods = [
      "all",
      "get",
      "head",
      "post",
      "patch",
      "put",
      "delete",
      "connect",
      "options",
      "trace"
    ];
    const invalid = methods.find(
      (method) => !allowedMethods.includes(method.toLowerCase())
    );
    if (invalid) {
      throw new Error(`${invalid} is not a valid HTTP method`);
    }
    const newMethods = methods.reduce((acc, method) => {
      acc[method.toLowerCase()] = !!options.methods[method];
      return acc;
    }, {});
    return {
      ...options,
      methods: newMethods
    };
  }
  _createFiles() {
    const { files, paths } = this._options;
    const routePath = removeSlashes(paths.route, false, true);
    return files.reduce((acc, file) => {
      let src;
      let route;
      let headers;
      if (typeof file === "object") {
        ({ route, path: src, headers } = file);
      } else {
        src = file;
        route = file;
      }
      src = path.join(paths.source, src);
      route = removeSlashes(route, true, false);
      route = `${routePath}/${route}`;
      acc[route] = {
        path: src,
        route,
        headers: headers || {}
      };
      return acc;
    }, {});
  }
}
const staticsController = controllerCreator(
  ({ getMiddlewares, ...options } = {}) => (app) => {
    const router = app.getRouter();
    const ctrl = new StaticsController({
      inject: {
        sendFile: app.get("sendFile")
      },
      ...options
    });
    let useMiddlewares;
    if (getMiddlewares) {
      useMiddlewares = getMiddlewares(app).map((middleware) => {
        if ("middleware" in middleware) {
          return middleware.connect(app);
        }
        return middleware;
      }).filter(notUndefined);
    }
    return ctrl.addRoutes(router, useMiddlewares);
  }
);
export {
  StaticsController,
  staticsController
};
//# sourceMappingURL=statics.js.map