import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { get } from "@homer0/object-utils";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { deferred } from "@homer0/deferred";
import { providerCreator } from "../../utils";
class HTMLGenerator {
  constructor({
    inject: { config, logger, frontendFs, valuesService },
    ...options
  }) {
    __publicField(this, "_options");
    __publicField(this, "_config");
    __publicField(this, "_logger");
    __publicField(this, "_frontendFs");
    __publicField(this, "_valuesService");
    __publicField(this, "_fileReady", false);
    __publicField(this, "_fileDeferred");
    this._config = config;
    this._logger = logger;
    this._frontendFs = frontendFs;
    this._valuesService = valuesService;
    this._options = deepAssignWithOverwrite(
      {
        template: "index.tpl.html",
        file: "index.html",
        silent: false,
        deleteTemplateAfter: true,
        replacePlaceholder: /\{\{appConfig(?:uration)?\}\}/,
        placeholderExpression: /\{\{(.*?)\}\}/gi,
        variableName: "appConfig",
        configKeys: ["features", "version", "postMessagesPrefix"]
      },
      options
    );
    if (this._valuesService && typeof this._valuesService.getValues !== "function") {
      throw new Error("The HTMLGenerator values service must have a `getValues` method");
    }
  }
  whenReady() {
    return this._fileDeferred ? this._fileDeferred.promise : Promise.resolve();
  }
  async generateHTML() {
    if (this._fileReady)
      return void 0;
    if (this._fileDeferred)
      return this._fileDeferred.promise;
    this._fileDeferred = deferred();
    const { template, deleteTemplateAfter, file, silent } = this._options;
    try {
      const templateContents = await this._frontendFs.read(template);
      const values = await this._getValues();
      const html = this._processHTML(templateContents, values);
      await this._frontendFs.write(file, html);
      if (!silent) {
        this._logger.success(`The HTML file was successfully generated (${file})`);
      }
      if (deleteTemplateAfter) {
        await this._frontendFs.delete(template);
        if (!silent) {
          this._logger.info(`The HTML template was successfully removed (${template})`);
        }
      }
      this._fileReady = true;
      this._fileDeferred.resolve();
      this._fileDeferred = void 0;
      return void 0;
    } catch (error) {
      this._fileDeferred.reject(error);
      this._fileDeferred = void 0;
      if (!silent) {
        this._logger.error("There was an error while generating the HTML");
      }
      throw error;
    }
  }
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
  _getValues() {
    if (this._valuesService) {
      return this._valuesService.getValues(this._options);
    }
    const { configKeys } = this._options;
    if (configKeys && configKeys.length) {
      return Promise.resolve(this._config.get(configKeys));
    }
    return Promise.resolve({});
  }
  _processHTML(template, values) {
    const { replacePlaceholder, placeholderExpression, variableName } = this._options;
    const htmlObject = JSON.stringify(values);
    let code = template.replace(
      replacePlaceholder,
      `window.${variableName} = ${htmlObject}`
    );
    const matches = [];
    let match = placeholderExpression.exec(code);
    while (match) {
      const [string, value] = match;
      if (string && value) {
        matches.push({
          string,
          value
        });
      }
      match = placeholderExpression.exec(code);
    }
    matches.forEach((info) => {
      code = code.replace(info.string, String(get(values, info.value)));
    });
    return code;
  }
}
const htmlGeneratorProvider = providerCreator(
  (options = {}) => (app) => {
    const {
      serviceName = "htmlGenerator",
      valuesServiceName = "htmlGeneratorValues",
      ...rest
    } = options;
    app.set(
      serviceName,
      () => new HTMLGenerator({
        inject: {
          config: app.get("config"),
          logger: app.get("logger"),
          frontendFs: app.get("frontendFs"),
          valuesService: app.try(valuesServiceName)
        },
        ...rest
      })
    );
    app.once("afterStart", () => {
      app.get(serviceName).generateHTML();
    });
  }
);
export {
  HTMLGenerator,
  htmlGeneratorProvider
};
//# sourceMappingURL=htmlGenerator.js.map