import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import fetch from "node-fetch";
import urijs from "urijs";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { provider } from "../../utils";
class HTTP {
  constructor({ inject: { logger }, ...options }) {
    __publicField(this, "_logger");
    __publicField(this, "_options");
    this._logger = logger;
    this._options = deepAssignWithOverwrite(
      {
        logRequests: false
      },
      options
    );
    this.fetch = this.fetch.bind(this);
  }
  async fetch(url, options = {}) {
    let useURL = url;
    if (options.qs) {
      useURL = urijs(url).query(options.qs).toString();
    }
    const fetchOptions = {
      method: (options.method || "get").toUpperCase(),
      body: options.body
    };
    let defaultHeaders;
    if (options.req) {
      defaultHeaders = this.getCustomHeadersFromRequest(options.req);
      const ip = this.getIPFromRequest(options.req);
      if (ip) {
        defaultHeaders["x-forwarded-for"] = ip;
      }
    }
    const headers = {
      ...defaultHeaders,
      ...options.headers
    };
    if (Object.keys(headers).length) {
      fetchOptions.headers = this.normalizeHeaders(headers);
    }
    const { logRequests } = this._options;
    if (logRequests) {
      this._logRequest(useURL, fetchOptions);
    }
    const response = await fetch(useURL, fetchOptions);
    if (logRequests) {
      this._logResponse(response);
    }
    return response;
  }
  getIPFromRequest(req) {
    const headerValue = req.headers["x-forwarded-for"];
    if (headerValue) {
      return String(headerValue);
    }
    return req?.connection?.remoteAddress || req?.socket?.remoteAddress;
  }
  getCustomHeadersFromRequest(req, options = {}) {
    const { includeXForwardedHeaders = false } = options;
    return Object.keys(req.headers).reduce((acc, headerName) => {
      if (headerName.startsWith("x-") && (includeXForwardedHeaders || !headerName.startsWith("x-forwarded-"))) {
        acc[headerName] = req.headers[headerName];
      }
      return acc;
    }, {});
  }
  normalizeHeaders(headers) {
    return Object.keys(headers).reduce((acc, name) => {
      const newName = name.split("-").map((part) => part.replace(/^(\w)/, (_, letter) => letter.toUpperCase())).join("-");
      acc[newName] = headers[name];
      return acc;
    }, {});
  }
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
  _logRequest(url, options) {
    const { method, headers } = options;
    const prefix = "REQUEST> ";
    const lines = ["--->>", `${prefix}${method} ${url}`];
    if (headers) {
      Object.keys(headers).forEach((header) => {
        const value = headers[header];
        lines.push(`${prefix}${header}: ${value}`);
      });
    }
    if (options.body) {
      lines.push(`${prefix}body: "${options.body}"`);
    }
    this._logger.info(lines);
  }
  _logResponse(response) {
    const prefix = "RESPONSE> ";
    const lines = [
      "<<---",
      `${prefix}${response.url}`,
      `${prefix}status: ${response.status}`
    ];
    response.headers.forEach((value, header) => {
      lines.push(`${prefix}${header}: ${value}`);
    });
    this._logger.info(lines);
  }
}
const httpProvider = provider((app) => {
  app.set("http", () => {
    const config = app.getConfig();
    const logRequests = config.get("debug.logRequests") === true;
    return new HTTP({
      inject: {
        logger: app.get("logger")
      },
      logRequests
    });
  });
});
export {
  HTTP,
  httpProvider
};
//# sourceMappingURL=http.js.map